*DECK LNSR
      SUBROUTINE LNSR(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
*
*-----------------------------------------------------------------------
*
* single iteration for the line optimization of the objective function.
*
*Copyright:
* Copyright (C) 2019 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input/output
* NENTRY  number of LCM objects or files used by the operator.
* HENTRY  name of each LCM object or file:
*         HENTRY(1): creation or modification type(L_OPTIMIZE);
*         HENTRY(2): read-only type(L_FLUX);
*         HENTRY(3): read-only type(L_GPT).
* IENTRY  type of each LCM object or file:
*         =1 LCM memory object; =2 XSM file; =3 sequential binary file;
*         =4 sequential ascii file.
* JENTRY  access of each LCM object or file:
*         =0 the LCM object or file is created;
*         =1 the LCM object or file is open for modifications;
*         =2 the LCM object or file is open in read-only mode.
* KENTRY  LCM object address or file unit number.
*
* Reference:
*  W. H. Press et Al., Numerical Recipies in FORTRAN, Second Edition,
*  Sect. 9.7, Cambridge (1992).
*
*-----------------------------------------------------------------------
*
      USE GANLIB
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER      NENTRY,IENTRY(NENTRY),JENTRY(NENTRY)
      TYPE(C_PTR)  KENTRY(NENTRY)
      CHARACTER    HENTRY(NENTRY)*12
*----
*  LOCAL VARIABLES
*----
      PARAMETER (NSTATE=40,MAXINT=50)
      TYPE(C_PTR) IPGRAD
      CHARACTER TEXT12*12,HSIGN*12
      INTEGER ISTATE(NSTATE),CNVTST,DNVTST
      DOUBLE PRECISION OPTPRR(NSTATE)
      REAL FLOTT
      DOUBLE PRECISION DFLOTT,SR,VDM(3,4),DSAVE(9)
*----
*  ALLOCATABLE ARRAYS
*----
      DOUBLE PRECISION, ALLOCATABLE, DIMENSION(:) :: GP,V,Y,YGG,GGY
      DOUBLE PRECISION, ALLOCATABLE, DIMENSION(:) :: X,P,G,XMIN,XMAX
      DOUBLE PRECISION, ALLOCATABLE, DIMENSION(:,:) :: AA,GG
*----
*  PARAMETER VALIDATION.
*----
      IF((IENTRY(1).NE.1).AND.(IENTRY(1).NE.2)) CALL XABORT('LNSR: LCM'
     1 //' OBJECT EXPECTED AT LHS.')
      IF(JENTRY(1).EQ.2) CALL XABORT('LNSR: OPTIMIZE ENTRY IN CREATE O'
     1 //'R MODIFICATION MODE EXPECTED.')
      IPGRAD=KENTRY(1)
      CALL LCMGTC(IPGRAD,'SIGNATURE',12,1,HSIGN)
      IF(HSIGN.NE.'L_OPTIMIZE') THEN
         TEXT12=HENTRY(1)
         CALL XABORT('LNSR: SIGNATURE OF '//TEXT12//' IS '//HSIGN//
     1   '. L_OPTIMIZE EXPECTED.')
      ENDIF
      CNVTST=-1
      ICONV =0
      DNVTST=-1
      JCONV =0
*----
*  READ INPUT PARAMETERS
*----
      CALL LCMGET(IPGRAD,'STATE-VECTOR',ISTATE)
      IF(ISTATE(2).NE.0) CALL XABORT('LNSR: CONSTRAINTS NOT IMPLEMENTE'
     1 //'D.')
      NVAR  =ISTATE(1)
      IOPT  =ISTATE(3)
      ICONV =ISTATE(4)
      IF((IOPT.NE.1).AND.(IOPT.NE.-1)) CALL XABORT('LNSR: IOPT not equ'
     1  //'al to 1 or -1')
      IEXT  =ISTATE(5)
      IEDSTP=ISTATE(6)
      IHESS =ISTATE(7)
      ISTEP =ISTATE(10)
      JCONV =ISTATE(11)
      MAXEXT=ISTATE(12)
      CALL LCMGET(IPGRAD,'OPT-PARAM-R',OPTPRR)
      SR=OPTPRR(1)
      EPS1=OPTPRR(2)
      EPS2=OPTPRR(3)
      EPS3=OPTPRR(4)
      IPICK=0
      IPRINT=1
   10 CALL REDGET(INDIC,NITMA,FLOTT,TEXT12,DFLOTT)
      IF(INDIC.EQ.10) GO TO 20
      IF(INDIC.NE.3) CALL XABORT('LNSR: CHARACTER DATA EXPECTED(1).')
      IF(TEXT12.EQ.'EDIT') THEN
        CALL REDGET(INDIC,IPRINT,FLOTT,TEXT12,DFLOTT)
        IF(INDIC.NE.1) CALL XABORT('LNSR: INTEGER DATA EXPECTED(1).')
      ELSE IF(TEXT12.EQ.'MINIMIZE') THEN
        IOPT=1
      ELSE IF(TEXT12.EQ.'MAXIMIZE') THEN
        IOPT=-1
      ELSE IF(TEXT12.EQ.'OUT-STEP-LIM') THEN
*       Set maximum step for line optimization.
        CALL REDGET(INDIC,NITMA,FLOTT,TEXT12,DFLOTT)
        IF(INDIC.EQ.2) THEN
          SR=FLOTT
        ELSE IF(INDIC.EQ.4) THEN
          SR=DFLOTT
        ELSE
          CALL XABORT('LNSR: REAL OR DOUBLE PRECISION VALUE EXPECTED.')
        ENDIF
      ELSE IF(TEXT12.EQ.'INN-STEP-EPS') THEN
*       Set the tolerence used for line optimization.
        CALL REDGET(INDIC,NITMA,FLOTT,TEXT12,DFLOTT)
        IF(INDIC.EQ.2) THEN
          EPS3=FLOTT
        ELSE IF(INDIC.EQ.4) THEN
          EPS3=DFLOTT
        ELSE
          CALL XABORT('LNSR: REAL OR DOUBLE PRECISION VALUE EXPECTED.')
        ENDIF
      ELSE IF(TEXT12.EQ.'OUT-STEP-EPS') THEN
*       Set the tolerence used for external iterations.
        CALL REDGET(INDIC,NITMA,FLOTT,TEXT12,DFLOTT)
        IF(INDIC.EQ.2) THEN
          EPS2=FLOTT
        ELSE IF(INDIC.EQ.4) THEN
          EPS2=DFLOTT
        ELSE
          CALL XABORT('LNSR: REAL OR DOUBLE PRECISION VALUE EXPECTED.')
        ENDIF
      ELSE IF(TEXT12.EQ.'OUT-ITER-MAX') THEN
        CALL REDGET(INDIC,MAXEXT,FLOTT,TEXT12,DFLOTT)
        IF(INDIC.NE.1) CALL XABORT('LNSR: INTEGER DATA EXPECTED(2).')
      ELSE IF(TEXT12.EQ.'SD') THEN
        IHESS=0
      ELSE IF(TEXT12.EQ.'CG') THEN
        IHESS=1
      ELSE IF(TEXT12.EQ.'BFGS') THEN
        IHESS=2
      ELSE IF(TEXT12.EQ.'INN-CONV-TST') THEN
*       Internal convergence test
        IPICK=1
        GO TO 20
      ELSE IF(TEXT12(:1).EQ.';') THEN
        GO TO 20
      ELSE 
        CALL XABORT('LNSR: '//TEXT12//' IS AN INVALID KEYWORD.')
      ENDIF
      GO TO 10
*----
*  RECOVER INFORMATION FROM OPTIM OBJECT
*----
   20 ISTEP=ISTEP+1
      ALLOCATE(X(NVAR),P(NVAR),G(NVAR),XMIN(NVAR),XMAX(NVAR))
      CALL LCMGET(IPGRAD,'FOBJ-CST-VAL',F)
      CALL LCMGET(IPGRAD,'VAR-VALUE',X)
      CALL LCMGET(IPGRAD,'GRADIENT',G)
      CALL LCMGET(IPGRAD,'VAR-VAL-MIN',XMIN)
      CALL LCMGET(IPGRAD,'VAR-VAL-MAX',XMAX)
      CALL LCMLEN(IPGRAD,'LNSR-INFO',ILONG,ITYLCM)
      IF(ILONG.NE.0) THEN
        CALL LCMGET(IPGRAD,'LNSR-INFO',DSAVE)
        SLOPE=DSAVE(1)
        ALAMIN=DSAVE(2)
        ALAM=DSAVE(3)
        ALAM2=DSAVE(4)
        ALAM3=DSAVE(5)
        F2=DSAVE(6)
        F3=DSAVE(7)
        ALPHA_OLD=DSAVE(8)
        GNORM=DSAVE(9)
      ELSE
        SLOPE=0.0D0
        ALAMIN=0.0D0
        ALAM=0.0D0
        ALAM2=0.0D0
        ALAM3=0.0D0
        F2=0.0D0
        F3=0.0D0
        ALPHA_OLD=0.0D0
        GNORM=0.0D0
      ENDIF
*----
*  SET THE DIRECTION AND INITIALIZATION OF THE LINE SEARCH
*----
      IF(ISTEP.EQ.1) THEN
        IF(IPRINT.GT.0) WRITE(6,100) IEXT,F
        IF(IHESS.EQ.0) THEN
*         Steepest descent
          P(:NVAR)=-G(:NVAR)
        ELSE IF(IHESS.EQ.1) THEN
          CALL LCMLEN(IPGRAD,'DIRECTION',ILONG,ITYLCM)
          IF(ILONG.EQ.0) THEN
*           Steepest descent
            P(:NVAR)=-G(:NVAR)
            GNORM=DOT_PRODUCT(G(:NVAR),G(:NVAR))/REAL(NVAR)
          ELSE
*           Conjugate gradient
            GNORMP=GNORM
            CALL LCMGET(IPGRAD,'DIRECTION',P)
            GNORM=DOT_PRODUCT(G(:NVAR),G(:NVAR))/REAL(NVAR)
            P(:NVAR)=-G(:NVAR)+(GNORM/GNORMP)*P(:NVAR)
          ENDIF
        ELSE IF(IHESS.EQ.2) THEN
*         BFGS
          CALL LCMLEN(IPGRAD,'HESSIAN',ILONG,ITYLCM)
          IF(ILONG.EQ.0) THEN
*           Steepest descent
            P(:NVAR)=-G(:NVAR)
          ELSE
*           Newton search
            ALLOCATE(GG(NVAR,NVAR))
            CALL LCMGET(IPGRAD,'HESSIAN',GG)
            P(:NVAR)= 0.0D0
            DO I=1,NVAR
              P(:NVAR)=P(:NVAR)-GG(:NVAR,I)*G(I)
            ENDDO
            DEALLOCATE(GG)
          ENDIF
        ENDIF
        GNORM=DOT_PRODUCT(G(:NVAR),G(:NVAR))/REAL(NVAR)
        PABS=SQRT(DOT_PRODUCT(P(:NVAR),P(:NVAR)))
        IF(PABS.GT.SR) P(:NVAR)=P(:NVAR)*SR/PABS
        SLOPE=DOT_PRODUCT(G(:NVAR),P(:NVAR))
        ALAMIN=EPS3/MAXVAL(ABS(P(:NVAR))/MAX(ABS(X(:NVAR)),1.0D0))
        ALAM=1.0D0
        IF(IOPT.EQ.-1) F=-F
        FOLD=F
        ALAM2=0.0D0
        F2=F
        ALAM3=0.0D0
        F3=F
        CALL LCMPUT(IPGRAD,'DIRECTION',NVAR,4,P)
        CALL LCMSIX(IPGRAD,'OLD-VALUE',1)
          CALL LCMPUT(IPGRAD,'VAR-VALUE',NVAR,4,X)
          CALL LCMPUT(IPGRAD,'FOBJ-CST-VAL',1,4,FOLD)
        CALL LCMSIX(IPGRAD,' ',2)
        GO TO 30
      ELSE
*       recover values at beginning of line search
        CALL LCMGET(IPGRAD,'DIRECTION',P)
        CALL LCMSIX(IPGRAD,'OLD-VALUE',1)
          CALL LCMGET(IPGRAD,'VAR-VALUE',X)
          CALL LCMGET(IPGRAD,'FOBJ-CST-VAL',FOLD)
          IF(IOPT.EQ.-1) FOLD=-FOLD
        CALL LCMSIX(IPGRAD,' ',2)
      ENDIF
*----
*  SINGLE INNER ITERATION OF THE LINE OPTIMIZATION
*----
      IF(IOPT.EQ.-1) F=-F
      IF(ABS(ALAM).LT.ALAMIN) THEN
        DNVTST=1
        JCONV =1
        GO TO 40
      ELSE IF(ABS(ALAM-ALAM2).LT.ALAMIN) THEN
        DNVTST=1
        JCONV =1
        GO TO 40
      ELSE IF(ABS(F-F2).LT.EPS3*ABS(FOLD)) THEN
        DNVTST=1
        JCONV =1
        GO TO 40
      ELSE IF(ISTEP.GT.MAXINT) THEN
        DNVTST=1
        JCONV =1
        GO TO 40
      ELSE
        IF(ALAM.EQ.1.0D0) THEN
          TMPLAM=-SLOPE/(2.0D0*(F-FOLD-SLOPE))
        ELSE
          VDM(:3,1)=1.0D0
          VDM(1,2)=ALAM ; VDM(1,3)=ALAM**2 ; VDM(1,4)=F ;
          VDM(2,2)=ALAM2 ; VDM(2,3)=ALAM2**2 ; VDM(2,4)=F2 ;
          VDM(3,2)=ALAM3 ; VDM(3,3)=ALAM3**2 ; VDM(3,4)=F3 ;
          CALL ALSBD(3,1,VDM,IER,3)
          IF(IER.NE.0) CALL XABORT('LNSR: SINGULAR MATRIX.')
          TMPLAM=-VDM(2,4)/(2.0D0*VDM(3,4))
          IF(ISTEP.GE.15) THEN
*           prevent oscillations.
            IF((F.LE.F2).AND.(F.LE.F3)) THEN
              TMPLAM=0.5D0*(ALAM+TMPLAM)
            ELSE IF((F2.LE.F).AND.(F2.LE.F3)) THEN
              TMPLAM=0.5D0*(ALAM2+TMPLAM)
            ELSE
              TMPLAM=0.5D0*(ALAM3+TMPLAM)
            ENDIF
          ENDIF
        ENDIF
      ENDIF
      TMPLAM=MAX(0.0D0,MIN(1.0D0,TMPLAM))
      ALAM3=ALAM2
      F3=F2
      ALAM2=ALAM
      F2=F
      ALAM=MAX(TMPLAM,0.1D0*ALAM)
   30 X(:NVAR)=X(:NVAR)+ALAM*P(:NVAR)
      DO I=1,NVAR
        X(I)=MAX(XMIN(I),MIN(XMAX(I),X(I)))
      ENDDO
      CALL LCMPUT(IPGRAD,'VAR-VALUE',NVAR,4,X)
   40 DEALLOCATE(XMAX,XMIN,G,P,X)
      IF(IPRINT.GT.0) WRITE(6,110) ISTEP,ALAM,F,JCONV
      IF(IPRINT.GT.2) THEN
        ALLOCATE(X(NVAR),P(NVAR))
        CALL LCMGET(IPGRAD,'DIRECTION',P)
        CALL LCMGET(IPGRAD,'VAR-VALUE',X)
        WRITE(6,120) '    LINE SEARCH DIRECTION',(P(I),I=1,NVAR)
        WRITE(6,120) 'OUTPUT DECISION VARIABLES',(X(I),I=1,NVAR)
        DEALLOCATE(P,X)
      ENDIF
*----
*  TEST FOR EXTERNAL ITERATION CONVERGENCE
*----
      IF((JCONV.EQ.1).AND.(ISTEP.GT.1)) THEN
        TEST2=ABS(F-FOLD)
        IF(ABS(ALAM).LT.EPS2) THEN
          IF(IPRINT.GT.0) PRINT *,'>>> OUTER CONVERGED WRT ALAM'
          CNVTST=1
          ICONV =1
        ELSE IF(GNORM.LT.EPS2) THEN
          IF(IPRINT.GT.0) PRINT *,'>>> OUTER CONVERGED WRT GNORM'
          CNVTST=1
          ICONV =1
        ELSE IF(TEST2.LT.EPS2) THEN
          IF(IPRINT.GT.0) PRINT *,'>>> OUTER CONVERGED WRT F-FOLD'
          CNVTST=1
          ICONV =1
        ELSE IF(IEXT.GE.MAXEXT) THEN
          IF(IPRINT.GT.0) PRINT *,'>>> OUTER REACHES MAXIMUM ITERATION'
          CNVTST=1
          ICONV =1
        ENDIF
        IEXT=IEXT+1
        IF(IPRINT.GT.0) WRITE(6,130) IEXT,ABS(ALAM),GNORM,TEST2,EPS2
*----
*  CALCULATION OF THE BFGS HESSIAN
*----
        ALLOCATE(G(NVAR),P(NVAR))
        CALL LCMGET(IPGRAD,'GRADIENT',G)
        IF(IHESS.EQ.2) THEN
*         Broyden-Fletcher-Goldfarb-Shanno method
          CALL LCMLEN(IPGRAD,'HESSIAN',ILONG,ITYLCM)
          IF(ILONG.EQ.NVAR*NVAR) THEN
            ALLOCATE(GG(NVAR,NVAR),X(NVAR),GP(NVAR),V(NVAR),Y(NVAR),
     1      GGY(NVAR),YGG(NVAR),AA(NVAR,NVAR))
            CALL LCMGET(IPGRAD,'HESSIAN',GG)
            CALL LCMGET(IPGRAD,'VAR-VALUE',X)
            CALL LCMSIX(IPGRAD,'OLD-VALUE',1)
              CALL LCMGET(IPGRAD,'GRADIENT',GP)
              CALL LCMGET(IPGRAD,'DIRECTION',P)
            CALL LCMSIX(IPGRAD,' ',2)
            V(:NVAR)=ALPHA_OLD*P(:NVAR)
            Y(:NVAR)=G(:NVAR)-GP(:NVAR)
            SVY=DOT_PRODUCT(V(:NVAR),Y(:NVAR))
            IF(SVY.EQ.0.0D0) CALL XABORT('LNSR: DIVIDE CHECK IN BFGS.')
            SVYI=1.0D0/SVY
            DO I=1,NVAR
              TMP1=0.0D0
              TMP2=0.0D0
              DO J=1,NVAR
                AA(J,I)=V(J)*V(I)*SVYI
                TMP1=TMP1+GG(I,J)*Y(J)
                TMP2=TMP2+Y(J)*GG(J,I)
              ENDDO
              GGY(I)=TMP1
              YGG(I)=TMP2
            ENDDO
            B=1.0D0
            DO I=1,NVAR
              B=B+Y(I)*GGY(I)*SVYI
            ENDDO
            AA(:NVAR,:NVAR)=AA(:NVAR,:NVAR)*B
            DO J=1,NVAR
              DO I=1,NVAR
                AA(I,J)=AA(I,J)-(V(I)*YGG(J)+GGY(I)*V(J))*SVYI
              ENDDO
            ENDDO
            GG(:NVAR,:NVAR)=GG(:NVAR,:NVAR)+AA(:NVAR,:NVAR)
            DEALLOCATE(AA,YGG,GGY,Y,V,GP,X)
          ELSE
            ALLOCATE(GG(NVAR,NVAR))
            GG(:NVAR,:NVAR)=0.0D0
            DO I=1,NVAR
              GG(I,I)=1.0D0
            ENDDO
          ENDIF
          CALL LCMPUT(IPGRAD,'HESSIAN',NVAR*NVAR,4,GG)
          DEALLOCATE(GG)
        ENDIF
        CALL LCMGET(IPGRAD,'DIRECTION',P)
        CALL LCMSIX(IPGRAD,'OLD-VALUE',1)
          CALL LCMPUT(IPGRAD,'GRADIENT',NVAR,4,G)
          CALL LCMPUT(IPGRAD,'DIRECTION',NVAR,4,P)
        CALL LCMSIX(IPGRAD,' ',2)
        DEALLOCATE(P,G)
        ALPHA_OLD=ALAM
      ENDIF
*----
*  SAVE THE STATE VECTORS
*----
      CALL XDISET(ISTATE,NSTATE,0)
      ISTATE(1)=NVAR
      ISTATE(3)=IOPT
      ISTATE(4)=ICONV
      ISTATE(5)=IEXT
      ISTATE(6)=IEDSTP
      ISTATE(7)=IHESS
      ISTATE(8)=2
      ISTATE(10)=ISTEP
      ISTATE(11)=JCONV
      ISTATE(12)=MAXEXT
      IF(IPRINT.GT.0) WRITE(6,140) (ISTATE(I),I=1,12)
      CALL LCMPUT(IPGRAD,'STATE-VECTOR',NSTATE,1,ISTATE)
      CALL XDDSET(OPTPRR,NSTATE,0.0D0)
      OPTPRR(1)=SR
      OPTPRR(2)=EPS1
      OPTPRR(3)=EPS2
      OPTPRR(4)=EPS3
      IF(IPRINT.GT.0) WRITE(6,150) (OPTPRR(I),I=1,4)
      CALL LCMPUT(IPGRAD,'OPT-PARAM-R',NSTATE,4,OPTPRR)
      DSAVE(1)=SLOPE
      DSAVE(2)=ALAMIN
      DSAVE(3)=ALAM
      DSAVE(4)=ALAM2
      DSAVE(5)=ALAM3
      DSAVE(6)=F2
      DSAVE(7)=F3
      DSAVE(8)=ALPHA_OLD
      DSAVE(9)=GNORM
      CALL LCMPUT(IPGRAD,'LNSR-INFO',9,4,DSAVE)
      IF(IPRINT.GT.2) CALL LCMLIB(IPGRAD)
*----
*  RECOVER THE CONVERGENCE FLAGS AND SAVE IT IN A CLE-2000 VARIABLE
*----
      IF(IPICK.EQ.1) THEN
        CALL REDGET(INDIC,NITMA,FLOTT,TEXT12,DFLOTT)
        IF(INDIC.NE.-5) CALL XABORT('LNSR: OUTPUT LOGICAL EXPECTED.')
        INDIC=5
        CALL REDPUT(INDIC,DNVTST,FLOTT,TEXT12,DFLOTT)
   50   CALL REDGET(INDIC,NITMA,FLOTT,TEXT12,DFLOTT)
        IF(INDIC.NE.3) CALL XABORT('LNSR: CHARACTER DATA EXPECTED(2).')
        IF(TEXT12.EQ.'OUT-CONV-TST') THEN
          CALL REDGET(INDIC,NITMA,FLOTT,TEXT12,DFLOTT)
          IF(INDIC.NE.-5) CALL XABORT('LNSR: OUTPUT LOGICAL EXPECTED.')
          INDIC=5
          CALL REDPUT(INDIC,CNVTST,FLOTT,TEXT12,DFLOTT)
          GO TO 50
        ELSE IF (TEXT12.EQ.';') THEN
          RETURN
        ELSE
          CALL XABORT('LNSR: ; CHARACTER EXPECTED.')
        ENDIF      
      ENDIF      
      RETURN
*
  100 FORMAT(/14H LNSR: ##ITER=,I8,20H OBJECTIVE FUNCTION=,1P,E14.6)
  110 FORMAT(/29H LNSR: LINE SEARCH ITERATION=,I4,7H ALPHA=,1P,
     1 E17.10,20H OBJECTIVE FUNCTION=,E17.10,6H CONV=,I2)
  120 FORMAT(/7H LNSR: ,A25,1H=,1P,8E12.4/(33X,8E12.4))
  130 FORMAT(/26H LNSR: EXTERNAL ITERATION=,I4,12H ACCURACIES=,1P,
     1 3E12.4,6H EPS2=,E12.4)
  140 FORMAT(/8H OPTIONS/8H -------/
     1 7H NVAR  ,I8,32H   (NUMBER OF CONTROL VARIABLES)/
     2 7H NCST  ,I8,26H   (NUMBER OF CONSTRAINTS)/
     3 7H IOPT  ,I8,37H   (=1/-1: MINIMIZATION/MAXIMIZATION)/
     4 7H ICONV ,I8,43H   (=0/1: EXTERNAL NOT CONVERGED/CONVERGED)/
     5 7H IEXT  ,I8,32H   (INDEX OF EXTERNAL ITERATION)/
     6 7H IEDSTP,I8,13H   (NOT USED)/
     7 7H IHESS ,I8,29H   (=0/1/2: STEEPEST/CG/BFGS)/
     8 7H ISEARC,I8,35H   (=0/1/2: NO SEARCH/OPTEX/NEWTON)/
     9 7H IMETH ,I8,13H   (NOT USED)/
     1 7H ISTEP ,I8,35H   (INDEX OF LINE SEARCH ITERATION)/
     2 7H JCONV ,I8,46H   (=0/1: LINE SEARCH NOT CONVERGED/CONVERGED)/
     3 7H MAXEXT,I8,42H   (MAXIMUM NUMBER OF EXTERNAL ITERATIONS))
  150 FORMAT(/
     1 12H REAL PARAM:,1P/12H -----------/
     2 7H SR    ,D12.4,33H   (MAXIMUM LINE SEARCH STEPSIZE)/
     3 7H EPS1  ,D12.4,13H   (NOT USED)/
     4 7H EPS2  ,D12.4,31H   (EXTERNAL CONVERGENCE LIMIT)/
     5 7H EPS3  ,D12.4,31H   (INTERNAL CONVERGENCE LIMIT))
      END
